void mode_dealloc(GGIModeObject *self) {
	PyObject_Del(self);
}

int mode_cmp(GGIModeObject *a, GGIModeObject *b) {
	if((a->mode.frames==b->mode.frames) &&
	   (a->mode.virt.x==b->mode.virt.x) &&
	   (a->mode.virt.y==b->mode.virt.y) &&
	   (a->mode.visible.x==b->mode.visible.x) &&
	   (a->mode.visible.y==b->mode.visible.y) &&
	   (a->mode.size.x==b->mode.size.x) &&
	   (a->mode.size.y==b->mode.size.y) &&
	   (a->mode.dpp.x==b->mode.dpp.x) &&
	   (a->mode.dpp.y==b->mode.dpp.y) &&
	   (a->mode.graphtype==b->mode.graphtype))
		return 0;
	return -1;
}

PyObject* mode_getattr(GGIModeObject *self, char *name) {
	switch(name[0]) {
	case 'f': 
		if(!strcmp(name,"frames"))
			return PyInt_FromLong(self->mode.frames);
		break;
	case 'v':
		if(!strcmp(name,"visible"))
			return Py_BuildValue("ii",self->mode.visible.x,
					     self->mode.visible.y);
		else if(!strcmp(name,"virt"))
			return Py_BuildValue("ii",self->mode.virt.x,
					     self->mode.virt.y);
		break;
	case 's':
		if(!strcmp(name,"size"))
			return Py_BuildValue("ii",self->mode.size.x,
					     self->mode.size.y);
		break;
	case 'g':
		/* TODO : Improve graphtype */
		if(!strcmp(name,"graphtype")==0)
			return PyInt_FromLong(self->mode.graphtype);
		break;
	case 'd':
		if(strcmp(name,"dpp")==0)
			return Py_BuildValue("ii",self->mode.dpp.x,
					     self->mode.dpp.y);
		break;
	}
	PyErr_SetString(PyExc_AttributeError, name);
	return NULL;
}

int mode_setattr(GGIModeObject *self, char *name, PyObject *val) {
	switch(name[0]) {
	case 'f': 
		if(!strcmp(name,"frames"))
			return pyggi_get_int(val,&self->mode.frames);
		break;
	case 'v':
		if(!strcmp(name,"visible"))
			return pyggi_get_coords(val,&self->mode.visible);
		if(!strcmp(name,"virt"))
			return pyggi_get_coords(val,&self->mode.virt);
		break;
	case 's':
		if(!strcmp(name,"size"))
			return pyggi_get_coords(val,&self->mode.size);		
		break;
	case 'g':
		if(!strcmp(name,"graphtype"))
			return pyggi_get_int(val,&self->mode.graphtype);
		break;
	case 'd':
		if(!strcmp(name,"dpp"))
			return pyggi_get_coords(val,&self->mode.dpp);		
		break;
	} 
	PyErr_SetString(PyExc_AttributeError, name);
	return -1;
}

static PyObject * mode_str(GGIModeObject * self) {
	char s[80];
	ggiSPrintMode(s, &(self->mode));
	return PyString_FromString(s);
}

static PyTypeObject mode_type = {
	PyObject_HEAD_INIT(NULL)
	0,
	"GGI Mode",
	sizeof(GGIModeObject),
	0,
	(destructor)mode_dealloc,
	(printfunc)0,
	(getattrfunc)mode_getattr, 
	(setattrfunc)mode_setattr,
	(cmpfunc)mode_cmp,
	0,

	0,
	0,
	0,

	0,
	0,
	(reprfunc)mode_str,

	0L,0L,0L,0L,
	"no docs."
};

PyObject* newModeFromGGIMode(ggi_mode m) {
	GGIModeObject *newmode;
	if((newmode=PyObject_New(GGIModeObject, &mode_type))==NULL)
		return NULL;
	newmode->mode=m;
	return (PyObject*)newmode;
}

const char mode_construct_doc[]="
Create a new mode object. 

Parameters : ()|(s) : An optional string describing the mode, as defined in GGI.
Return:      O : A new Mode object. If the function is called from a visual object,
                 the mode is checked against that visual after being initialized.
";
PyObject* ModeConstruct(GGIVisualObject *self, PyObject*args, PyObject*keys) {
	ggi_mode m;
	char *c;
/*	PyObject *foo; */

	m.frames=GGI_AUTO;
	m.graphtype=GGI_AUTO;
	m.visible.x=GGI_AUTO;
	m.visible.y=GGI_AUTO;
	m.virt.x=GGI_AUTO;
	m.virt.y=GGI_AUTO;
	m.dpp.x=GGI_AUTO;
	m.dpp.y=GGI_AUTO;
	m.size.x=GGI_AUTO;
	m.size.y=GGI_AUTO;

	switch(PyTuple_Size(args)) {
	case 0:
		break;
	case 1:		
		if(!PyString_Check(PyTuple_GET_ITEM(args,0))) {
			PyErr_SetString(PyExc_TypeError, "string required");
			return NULL;
		}
		c=PyString_AsString(PyTuple_GET_ITEM(args,0));
		if (PyErr_Occurred())
					 return NULL;
		ggiParseMode(c,&m);
		break;
	default:
		PyErr_SetString(PyExc_TypeError, "Mode requires at most one arg");
		return NULL;
	}
	
	/* check the dictionnary
	if(keys && PyDict_Check(keys)) {
		if((foo=PyDict_GetItemString(keys,"visible")))
			if(_parse_coord(&m.visible.x,&m.visible.y))
				return NULL;
		if((foo=PyDict_GetItemString(keys,"virt")))
			if(_parse_coord(&m.virt.x,&m.virt.y))
				return NULL;
		if((foo=PyDict_GetItemString(keys,"size")))
			if(_parse_coord(&m.size.x,&m.size.y))
				return NULL;
		if((foo=PyDict_GetItemString(keys,"dpp")))
			if(_parse_coord(&m.dpp.x,&m.dpp.y))
				return NULL;
		if((foo=PyDict_GetItemString(keys,"frames"))) {
			if(!PyInt_Check(val)) {
				PyErr_SetString(PyExc_TypeError, "int expected");
				return NULL;
			}
			self->mode.frames=PyInt_AsLong(val);
		}
		if((foo=PyDict_GetItemString(keys,"graphtype"))) {
			if(!PyInt_Check(val)) {
				PyErr_SetString(PyExc_TypeError, "int expected");
				return NULL;
			}
			self->mode.graphtype=PyInt_AsLong(val);
		}
	}
	*/

	if(self!=NULL) {
		/* Call from a visual object. check the mode */
		ggiCheckMode(self->vis, &m);
	}
	
	return newModeFromGGIMode(m);
}
